///////////////////////////////////////////////////////////////////////////
// pamcomp.dll v1.0                                 (c) 1999 Dirk Taeger //
///////////////////////////////////////////////////////////////////////////
// DLL to compute person-years and death distribution. You need          //
// VC++ 6.0 SP3 to compile this DLL. This DLL is designed to be used     //
// with the PAMCOMP software. See										 //
//																		 //	
// http://medweb.uni-muenster.de/institute/epi/pamcomp/pamcomp.html      //
//																		 // 
// for details.														     //		
///////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <stdlib.h>
#include <math.h>

#ifdef __cplusplus
#define EXPORT extern "C" __declspec(dllexport)
#else
#define EXPORT __declspec(dllexport)
#endif

// turns off all optimizations. Necessary otherwise this DLL will compute rubbish 'cause
// optimization is not done well with this compiler 
#pragma optimize("", off)


// prototypes
EXPORT double CALLBACK PYCOMPUTE(double dblEPS, double dblTPS, double dblDOB, double *py,int pyAnzahl, double *ageclass, short ageAnzahl, double *yearclass, short yearAnzahl);
EXPORT double CALLBACK DEATHCOMPUTE(double dblTPS, double dblDOB, double *dth,int dthAnzahl, double *ageclass, short ageAnzahl, double *yearclass, short yearAnzahl);


// main entry into the DLL
int WINAPI DllMain(HINSTANCE hInstance,DWORD fdwReason,PVOID pReserved)
{
    switch (fdwReason)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    } 
    return TRUE;
}


EXPORT double CALLBACK PYCOMPUTE(double dblEPS, double dblTPS, double dblDOB, double *py,int pyAnzahl, double *ageclass, short ageAnzahl, double *yearclass, short yearAnzahl)
{

// function to compute person-years
// will be called by the PAMCOMP software

// dblEPS     - entry-point-of-study
// dblDOB     - date-of-birth
// *py        - pointer to the array (one column) of the person-years
// pyAnzahl   - number of array elements
// *ageclass  - age classes
// ageAnzahl  - number of age classes
// *yearclass -	year classes     
// yearAnzahl - number of yearclasses

  double zeit;
  double age;
  int ageIndex=0;
  int calendarIndex=0;
  int blnAge=0;
  int blnCalendar=0;
  int i;
  int j;

  for(i=0;i<=pyAnzahl;i++)
	py[i]=0;


  for(zeit=dblEPS; zeit<=dblTPS;zeit++)
   {
     age = (zeit-dblDOB)/365.25;			// age computation

	 for(i=1; i<=ageAnzahl; i++)
	  {
	   if(age >= ageclass[i] && age < ageclass[i + 1])
		{
          ageIndex = i;            // py belonging to ageclass i
          blnAge = 1;
		  break;
	    }
      }

	  for(j=1; j<= yearAnzahl; j++)
	   {
	    if(zeit>= yearclass[j] && zeit< yearclass[j + 1])
		 {
          calendarIndex = j;	  // py belonging to yearclass j
          blnCalendar = 1;
          break;
	     }
       }


	   if(blnAge==1 && blnCalendar==1)
        {
         py[(calendarIndex-1)*ageAnzahl+ageIndex]++;  // increment py by one day
	     blnAge = 0;
         blnCalendar = 0;
	    }
    }

  return 1;

}


EXPORT double CALLBACK DEATHCOMPUTE(double dblTPS, double dblDOB, double *dth,int dthAnzahl, double *ageclass, short ageAnzahl, double *yearclass, short yearAnzahl)
{

// function to compute death distribution
// will be called by the PAMCOMP software

// dblTPS     - termination-point-of-study
// dblDOB     - date-of-birth
// *dth       - pointer to the array (one column) of deaths
// dthAnzahl  - number of array elements
// *ageclass  - age classes
// ageAnzahl  - number of age classes
// *yearclass -	year classes     
// yearAnzahl - number of yearclasses


  double age;
  int ageIndex=0;
  int calendarIndex=0;
  int blnAge=0;
  int blnCalendar=0;
  int i;
  int j;

  for(i=0;i<=dthAnzahl;i++)
	dth[i]=0;

  age = (dblTPS - dblDOB)/365.25;

   for(i=1; i<= (int)ageAnzahl; i++)
	{
	 if(age >= ageclass[i] && age < ageclass[i + 1])
	  {
       ageIndex = i;            // dead belongs to ageclass i
       blnAge = 1;
	   break;
	   }
     }

    for(j=1; j<= (int)yearAnzahl; j++)
     {
      if(dblTPS>= yearclass[j] && dblTPS < yearclass[j + 1])
       {
        calendarIndex = j;	   // dead belongs to yearclass j
        blnCalendar = 1;
        break;
	    }
      }


    if(blnAge==1 && blnCalendar==1)
     {
      dth[(calendarIndex-1)*ageAnzahl+ageIndex]++;
	  blnAge = 0;
      blnCalendar = 0;
     }

	return 1;
}
